#-*- coding: utf-8 -*-
"""
Backend for test environment.
"""

import sendsms
from sendsms.backends.base import BaseSmsBackend

class SmsBackend(BaseSmsBackend):
    """
    A sms backend for use during test sessions.

    The test connection stores messages in a dummy outbox,
    rather than sending them out on the wire.
    The dummy outbox is accessible through the outbox instance attribute.
    """
    def __init__(self, *args, **kwargs):
        super(SmsBackend, self).__init__(*args, **kwargs)
        if not hasattr(sendsms, 'outbox'):
            sendsms.outbox = []

    def send_messages(self, messages):
        """Redirect messages to the dummy outbox"""
        sendsms.outbox.extend(messages)
        return len(messages)



#-*- coding: utf-8 -*-
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.importlib import import_module
from sendsms.utils import load_object


def send_sms(body, from_phone, to, flash=False, fail_silently=False,
             auth_user=None, auth_password=None, connection=None):
    """
    Easy wrapper for send a single SMS to a recipient list. 

    :returns: the number of SMSs sent.
    """
    from sendsms.message import SmsMessage
    connection = connection or get_connection(
        username = auth_user, 
        password = auth_password,
        fail_silently = fail_silently
    )
    return SmsMessage(body=body, from_phone=from_phone, to=to, \
        flash=flash, connection=connection).send()


def send_mass_sms(datatuple, fail_silently=False,
             auth_user=None, auth_password=None, connection=None):
    """
    Given a datatuple of (message, from_phone, to, flash), sends each message to each
    recipient list. 
    
    :returns: the number of SMSs sent.
    """

    from sendsms.message import SmsMessage
    connection = connection or get_connection(
        username = auth_user, 
        password = auth_password,
        fail_silently = fail_silently
    )
    messages = [SmsMessage(message=message, from_phone=from_phone, to=to, flash=flash)
                for message, from_phonenumber, to, flash in datatuple]
    connection.send_messages(messages)


def get_connection(path=None, fail_silently=False, **kwargs):
    """
    Load an sms backend and return an instance of it.

    :param string path: backend python path. Default: sendsms.backends.console.SmsBackend
    :param bool fail_silently: Flag to not throw exceptions on error. Default: False
    :returns: backend class instance.
    :rtype: :py:class:`~sendsms.backends.base.BaseSmsBackend` subclass
    """

    path = path or getattr(settings, 'SENDSMS_BACKEND', 'sendsms.backends.locmem.SmsBackend')
    try:
        mod_name, klass_name = path.rsplit('.', 1)
        mod = import_module(mod_name)
    except AttributeError as e:
        raise ImproperlyConfigured(u'Error importing sms backend module %s: "%s"' % (mod_name, e))

    try:
        klass = getattr(mod, klass_name)
    except AttributeError:
        raise ImproperlyConfigured('Module "%s" does not define a "%s" class' % (mod_name, klass_name))

    return klass(fail_silently=fail_silently, **kwargs)



# -*- coding: utf-8 -*-
"""
Esendex sms gateway backend. (http://www.esendex.es/)

Configuration example.
~~~~~~~~~~~~~~~~~~~~~~

Modify your settings.py::
    
    ESENDEX_USERNAME = 'yourusername'
    ESENDEX_PASSWORD = 'mysecretpassword'
    ESENDEX_ACCOUNT = 'account-key-provided-by-esendex'
    ESENDEX_SANDBOX = False # True if yo like test first
    INSTALLED_APPS += ['sendsms']

Usage::
    
    from sendsms.message import SmsMessage
    message = SmsMessage(
        body = 'my 160 chars sms',
        from_phone = '111111111',
        to = ['222222222']
    )
    message.send()
"""

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.encoding import force_unicode

import requests

from .base import BaseSmsBackend

ESENDEX_API_URL = 'https://www.esendex.com/secure/messenger/formpost/SendSMS.aspx'
ESENDEX_USERNAME = getattr(settings, 'ESENDEX_USERNAME', '')
ESENDEX_PASSWORD = getattr(settings, 'ESENDEX_PASSWORD', '')
ESENDEX_ACCOUNT = getattr(settings, 'ESENDEX_ACCOUNT', '')
ESENDEX_SANDBOX = getattr(settings, 'ESENDEX_SANDBOX', False)

class SmsBackend(BaseSmsBackend):
    """ 
    SMS Backend for esendex.es provider.

    The methods "get_xxxxxx" serve to facilitate the inheritance. Thus if a private 
    project in the access data are dynamic, and are stored in the database. A child 
    class overrides the method "get_xxxx" to return data stored in the database.
    """

    def get_username(self):
        return ESENDEX_USERNAME

    def get_password(self):
        return ESENDEX_PASSWORD

    def get_account(self):
        return ESENDEX_ACCOUNT

    def _parse_response(self, response):
        """
        Parse http raw respone into python
        dictionary object.
        
        :param str response: http response
        :returns: response dict
        :rtype: dict
        """

        response_dict = {}
        for line in response.splitlines():
            key, value = response.split("=", 1)
            response_dict[key] = value
        return response_dict

    def _send(self, message):
        """
        Private method for send one message.

        :param SmsMessage message: SmsMessage class instance.
        :returns: True if message is sent else False
        :rtype: bool
        """

        params = {
            'EsendexUsername': self.get_username(),
            'EsendexPassword': self.get_password(),
            'EsendexAccount': self.get_account(), 
            'EsendexOriginator': message.from_phone, 
            'EsendexRecipient': ",".join(message.to),
            'EsendexBody': message.body,
            'EsendexPlainText':'1'
        }
        if ESENDEX_SANDBOX:
            params['EsendexTest'] = '1'

        response = requests.post(ESENDEX_API_URL, params)
        if response.status_code != 200:
            if not self.fail_silently:
                raise Exception('Bad status code')
            else:
                return False
        
        if not response.content.startswith('Result'):
            if not self.fail_silently:
                raise Exception('Bad result')
            else: 
                return False

        response = self._parse_response(response.content)
        
        if ESENDEX_SANDBOX and response['Result'] == 'Test':
            return True
        else:
            if response['Result'].startswith('OK'):
                return True
            else:
                if not self.fail_silently:
                    raise Exception('Bad result')
        
        return False

    def send_messages(self, messages):
        """
        Send messages.

        :param list messages: List of SmsMessage instences.
        :returns: number of messages seded succesful.
        :rtype: int
        """
        counter = 0
        for message in messages:
            res = self._send(message)
            if res:
                counter += 1

        return counter



#-*- coding: utf-8 -*-
from sendsms.api import get_connection
from sendsms.signals import sms_post_send
from django.conf import settings

class SmsMessage(object):
    """
    A sms message
    """
    def __init__(self, body, from_phone=None, to=None, flash=False, connection=None):
        """
        Initialize a single SMS message (which can be sent to multiple recipients)
        """
        if to:
            #assert not isinstance(to, basetring), '"to" argument must be a list or tuple'
            self.to = list(to)
        else:
            self.to = []

        self.from_phone = from_phone or getattr(settings, 'SENDSMS_DEFAULT_FROM_PHONE', '')
        self.body = body
        self.flash = flash
        self.connection = connection

    def get_connection(self, fail_silently=False):
        if not self.connection:
            self.connection = get_connection(fail_silently=fail_silently)
        return self.connection

    def send(self, fail_silently=False):
        """
        Sends the sms message
        """
        if not self.to:
            # Don't bother creating the connection if there's nobody to send to
            return 0
        res = self.get_connection(fail_silently).send_messages([self])
        sms_post_send.send(sender=self, to=self.to, from_phone=self.from_phone, body=self.body)
        return res




#-*- coding: utf-8 -*-
"""
Dummy sms backend that does nothing.
"""

from sendsms.backends.base import BaseSmsBackend

class SmsBackend(BaseSmsBackend):
    def send_messages(self, messages):
        return len(messages)



# -*- coding: utf-8 -*-

"""
SmsPubli sms gateway backend. (http://www.smspubli.com)

Configuration example.
----------------------

Modify your settings.py::

    SMSPUBLI_USERNAME = 'yourusername'
    SMSPUBLI_PASSWORD = 'mysecretpassword'
    SMSPUBLI_ALLOW_LONG_SMS = True # or False
    INSTALLED_APPS += ['sendsms']


Usage::

    from sendsms.message import SmsMessage
    message = SmsMessage(
        body = 'my 160 chars sms',
        from_phone = '111111111',
        to = ['222222222']
    )
    message.send()
"""

import requests, logging
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.encoding import force_unicode

from .base import BaseSmsBackend

SMSPUBLI_API_URL = 'https://secure.gateway360.com/api/push/'
SMSPUBLI_API_VERSION = 'HTTPV3'
SMSPUBLI_DC = 'SMS'
SMSPUBLI_DR = 0
SMSPUBLI_ROUTE = 2

SMSPUBLI_USERNAME = getattr(settings, 'SMSPUBLI_USERNAME', '')
SMSPUBLI_PASSWORD = getattr(settings, 'SMSPUBLI_PASSWORD', '')
SMSPUBLI_ALLOW_LONG_SMS = getattr(settings, 'SMSPUBLI_ALLOW_LONG_SMS', False)

class SmsBackend(BaseSmsBackend):
    """ 
    SMS Backend smspubli.com provider.

    The methods "get_xxxxxx" serve to facilitate the inheritance. Thus if a private 
    project in the access data are dynamic, and are stored in the database. A child 
    class overrides the method "get_xxxx" to return data stored in the database.
    """

    def get_username(self):
        return SMSPUBLI_USERNAME
    
    def get_password(self):
        return SMSPUBLI_PASSWORD

    def _send(self, message):
        """
        Private method for send one message.

        :param SmsMessage message: SmsMessage class instance.
        :returns: True if message is sended else False
        :rtype: bool
        """

        params = {
            'V': SMSPUBLI_API_VERSION, 
            'UN': SMSPUBLI_USERNAME, 
            'PWD': SMSPUBLI_PASSWORD,
            'R': SMSPUBLI_ROUTE, 
            'SA': message.from_phone,
            'DA': ','.join(message.to),
            'M': message.body.encode('latin-1'),
            'DC': SMSPUBLI_DC,
            'DR': SMSPUBLI_DR, 
            'UR': message.from_phone
        }
        if SMSPUBLI_ALLOW_LONG_SMS:
            params['LM'] = '1'

        response = requests.post(SMSPUBLI_API_URL, params)
        if response.status_code != 200:
            if  not self.fail_silently:
                raise
            else:
                return False

        response_msg, response_code = response.content.split(':')
        if response_msg == 'OK':
            try:
                if "," in response_code:
                    codes = map(int, response_code.split(","))
                else:
                    codes = [int(response_code)]

                for code in codes:
                    if code == -5:
                        #: TODO send error signal (no $$)
                        pass
                    elif code == -3:
                        #: TODO send error signal (incorrect num)
                        pass

                return True

            except (ValueError, TypeError):
                if not self.fail_silently:
                    raise
                return False
        
        return False

    def send_messages(self, messages):
        """
        Send messages.

        :param list messages: List of SmsMessage instences.
        :returns: number of messages seded succesful.
        :rtype: int
        """

        counter = 0
        for message in messages:
            res = self._send(message)
            if res:
                counter += 1

        return counter



#-*- coding: utf-8 -*-
"""
this backend requires the twilio python library: http://pypi.python.org/pypi/twilio/
"""
from twilio.rest import TwilioRestClient
from django.conf import settings
from sendsms.backends.base import BaseSmsBackend

TWILIO_ACCOUNT_SID = getattr(settings, 'SENDSMS_TWILIO_ACCOUNT_SID', '')
TWILIO_AUTH_TOKEN = getattr(settings, 'SENDSMS_TWILIO_AUTH_TOKEN', '')

class SmsBackend(BaseSmsBackend):
    def send_messages(self, messages):
        client = TwilioRestClient(TWILIO_ACCOUNT_SID, TWILIO_AUTH_TOKEN)
        for message in messages:
            for to in message.to:
                try:
                    msg = client.sms.messages.create(
                        to=to,
                        from_=message.from_phone,
                        body=message.body
                    )
                except:
                    if not self.fail_silently:
                        raise


#-*- coding: utf-8 -*-


class BaseSmsBackend(object):
    """
    Base class for sms backend implementations.

    Subclasses must at least overwrite send_messages()
    """
    def __init__(self, fail_silently=False, **kwargs):
        self.fail_silently = fail_silently

    def open(self):
        """
        Open a network connection.

        This method can be overwritten by backend implementations to open a network connection.
        It's up to the backend implementation to track the status of
        a network connection if it's needed by the backend.
        This method can be called by applications to force a single
        network connection to be used when sending multiple SMSs.

        The default implementation does nothing.
        """
        pass

    def close(self):
        """Close a network connection"""
        pass

    def send_messages(self, messages):
        """
        Sends one or more SmsMessage objects and returns the number of messages sent
        """
        raise NotImplementedError



from setuptools import setup, find_packages

version = __import__('sendsms').__version__

setup(
    name="django-sendsms",
    version=version,
    url='http://github.com/stefanfoulis/django-sendsms',
    license='BSD',
    platforms=['OS Independent'],
    description="A simple API to send SMS messages.",
    long_description=open('README.rst').read(),
    author='Stefan Foulis',
    author_email='stefan.foulis@gmail.com',
    maintainer='Stefan Foulis',
    maintainer_email='stefan.foulis@gmail.com',
    packages=find_packages(),
    include_package_data=True,
    extras_require = {
      'esendex': ['requests'],
      'twilio': ['twilio'],
      'smssluzbacz': ['smssluzbacz-api']
    },
    zip_safe=False,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
    ]
)



#    The New BSD License
#
#    Copyright (c) 2011-2012, CodeScale s.r.o.
#    All rights reserved.
#
#    Redistribution and use in source and binary forms, with or without modification,
#    are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#    * Neither the name of CodeScale s.r.o. nor the names of its contributors
#    may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
#    INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#    DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
#    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
#    WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#    USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import logging
import unicodedata

from smssluzbacz_api.lite import SmsGateApi
from django.conf import settings

from sendsms.backends.base import BaseSmsBackend


log = logging.getLogger(__name__)


class SmsBackend(BaseSmsBackend):
    """SmsBackend for sms.sluzba.cz API.

    settings.py configuration constants:
      SMS_SLUZBA_API_LOGIN - sms.sluzba.cz login
      SMS_SLUZBA_API_PASSWORD - sms.sluzba.cz password
      SMS_SLUZBA_API_TIMEOUT - connection timeout to sms.sluzba.cz in seconds
      SMS_SLUZBA_API_USE_SSL - whether to use ssl via http or not
      SMS_SLUZBA_API_USE_POST - whether to use GET or POST http method

    """

    def __init__(self, fail_silently=False, **kwargs):
        super(SmsBackend, self).__init__(fail_silently=fail_silently, **kwargs)
        self.open()

    def __del__(self):
        self.close()

    def open(self):
        """Initializes sms.sluzba.cz API library."""
        self.client = SmsGateApi(getattr(settings, 'SMS_SLUZBA_API_LOGIN', ''),
                                 getattr(settings, 'SMS_SLUZBA_API_PASSWORD', ''),
                                 getattr(settings, 'SMS_SLUZBA_API_TIMEOUT', 2),
                                 getattr(settings, 'SMS_SLUZBA_API_USE_SSL', True))

    def close(self):
        """Cleaning up the reference for sms.sluzba.cz API library."""
        self.client = None

    def send_messages(self, messages):
        """Sending SMS messages via sms.sluzba.cz API.

        Note:
          This method returns number of actually sent sms messages
          not number of SmsMessage instances processed.

        :param messages: list of sms messages
        :type messages: list of sendsms.message.SmsMessage instances
        :returns: number of sent sms messages
        :rtype: int

        """
        count = 0
        for message in messages:
            message_body = unicodedata.normalize('NFKD', unicode(message.body)).encode('ascii', 'ignore')
            for tel_number in message.to:
                try:
                    self.client.send(tel_number, message_body, getattr(settings, 'SMS_SLUZBA_API_USE_POST', True))
                except Exception:
                    if self.fail_silently:
                        log.exception('Error while sending sms via sms.sluzba.cz backend API.')
                    else:
                        raise
                else:
                    count += 1

        return count


#-*- coding: utf-8 -*-
from importlib import import_module

def load_object(import_path):
    """
    Shamelessly stolen from https://github.com/ojii/django-load
    
    Loads an object from an 'import_path', like in MIDDLEWARE_CLASSES and the
    likes.

    Import paths should be: "mypackage.mymodule.MyObject". It then imports the
    module up until the last dot and tries to get the attribute after that dot
    from the imported module.

    If the import path does not contain any dots, a TypeError is raised.

    If the module cannot be imported, an ImportError is raised.

    If the attribute does not exist in the module, a AttributeError is raised.
    """
    if '.' not in import_path:
        raise TypeError(
            "'import_path' argument to 'load_object' must "
            "contain at least one dot."
        )
    module_name, object_name = import_path.rsplit('.', 1)
    module = import_module(module_name)
    return getattr(module, object_name)



#-*- coding: utf-8 -*-
"""
SMS backend that writes messages to console instead of sending them.

This is a total ripoff of django.core.mail.backends.console
"""
import sys
import threading

from sendsms.backends.base import BaseSmsBackend

class SmsBackend(BaseSmsBackend):
    def __init__(self, *args, **kwargs):
        self.stream = kwargs.pop('stream', sys.stdout)
        self._lock = threading.RLock()
        super(SmsBackend, self).__init__(*args, **kwargs)

    def send_messages(self, messages):
        """Write all messages to the stream in a thread-safe way."""
        if not messages:
            return
        self._lock.acquire()
        try:
            # The try-except is nested to allow for
            # Python 2.4 support (Refs #12147)
            try:
                stream_created = self.open()
                for message in messages:
                    self.stream.write(render_message(message))
                    self.stream.write('\n')
                    self.stream.write('-'*79)
                    self.stream.write('\n')
                    self.stream.flush()  # flush after each message
                if stream_created:
                    self.close()
            except:
                if not self.fail_silently:
                    raise
        finally:
            self._lock.release()
        return len(messages)

def render_message(message):
    return u"""from: %(from)s\nto: %(to)s\nflash: %(flash)s\n%(body)s""" % {
        'from': message.from_phone,
        'to': ", ".join(message.to),
        'flash': message.flash,
        'body': message.body,
    }




class InvalidPhoneNumber(Exception):
    """
    The backends can use this exception to raise a standardized exception, if the
    backend returns an error code related to the phone number.
    """
    pass